<?php

namespace Drupal\config_enforce_devel;

use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\config_enforce\ConfigEnforce;

/**
 * A collection of enforced configs for a given target module.
 */
class EnforcedConfigCollection {

  use StringTranslationTrait, MessengerTrait, LoggerChannelTrait;

  const CONFIG_PREFIX = 'config_enforce.enforced_configs';

  // The machine name of the target module where this collection of enforced
  // configs is stored.
  protected $targetModule;

  // A logger service.
  protected $log;

  // A config factory.
  protected $configFactory;

  // An instance of the Drupal file system service.
  protected $fileSystem;

  /**
   * A basic constructor method.
   */
  public function __construct(string $target_module) {
    $this->log = $this->getLogger('config_enforce_devel::EnforcedConfigCollection');
    $this->configFactory = \Drupal::configFactory();
    $this->fileSystem = \Drupal::service('file_system');
    $this->setTargetModule($target_module);
  }

  /**
   * Set the target module for this enforced config.
   */
  public function setTargetModule(string $target_module) {
    $this->targetModule = $target_module;
    return $this;
  }

  /**
   * Return the target module for this enforced config.
   */
  public function getTargetModule() {
    return $this->targetModule;
  }

  /**
   * Return an editable instance of the module's stored enforced configs.
   */
  public function getEditableConfig() {
    return $this->configFactory->getEditable($this->getConfigName());
  }

  /**
   * Return the config name for the target module.
   */
  public function getConfigName() {
    return self::CONFIG_PREFIX . '.' . $this->getTargetModule();
  }

  /**
   * Return a list of all enforced config configs.
   */
  public static function getAllConfigNames() {
    return (new static(''))->getConfigNames();
  }

  /**
   * Return a list of all enforced config configs.
   */
  protected function getConfigNames() {
    return $this->configFactory->listAll(self::CONFIG_PREFIX);
  }

  /**
   * Delete this collection of enforced configs.
   */
  public function delete() {
    $config_file = ConfigEnforce::getConfigFilePath($this->getConfigName());
    $this->fileSystem->delete($config_file);
    $this->getEditableConfig()->delete();

    $module = $this->getTargetModule();
    $message = $this->t('Unregistered %name (@machine_name) as a target module.', [
      '%name' => (new TargetModuleCollection)->getTargetModuleLabel($module),
      '@machine_name' => $module,
    ]);
    $this->log->notice($message);
  }

}
