<?php

namespace Drupal\config_enforce_devel;

use Drupal\config_enforce\ConfigEnforce;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Logger\LoggerChannelTrait;

/**
 * A collection of all target modules.
 */
class TargetModuleCollection {

  use StringTranslationTrait, MessengerTrait, LoggerChannelTrait;

  // A logger service.
  protected $log;

  // A config factory.
  protected $configFactory;

  /**
   * A basic constructor method.
   */
  public function __construct() {
    $this->log = $this->getLogger('config_enforce_devel::TargetModuleCollection');
    $this->configFactory = \Drupal::configFactory();
  }

  /**
   * Return human-readable names for all target modules.
   */
  public function getTargetModuleLabels() {
    $modules = [];
    $available_modules = $this->getTargetModules();
    $available_modules = !is_null($available_modules) ? $available_modules : [];
    foreach ($available_modules as $module) {
      $modules[$module] = ConfigEnforce::getAllModules()[$module];
    }
    return $modules;
  }

  /**
   * Return a human-readable names for a target module.
   */
  public function getTargetModuleLabel(string $module) {
    return $this->getTargetModuleLabels()[$module];
  }


  /**
   * Return a list of available modules to target for writing config.
   */
  public function getTargetModules() {
    $target_modules = &drupal_static(__FUNCTION__);
    if (!isset($target_modules)) {
      $target_modules = [];
      foreach (EnforcedConfigCollection::getAllConfigNames() as $config) {
        $module = substr($config, strlen(EnforcedConfigCollection::CONFIG_PREFIX) + 1);
        // Something is generating a phantom config (`config_enforce.enforced_configs.`)
        // @TODO remove this once we no longer get the spurious config.
        if (empty($module)) continue;
        $target_modules[] = $module;
      }
    }
    return $target_modules;
  }

  /**
   * Ensure that associated enforced configs registries exist for given modules.
   */
  public function ensureTargetModulesAreRegistered($modules) {
    $current_target_modules = $this->getTargetModules();

    $new_target_modules = array_diff($modules, $current_target_modules);
    foreach ($new_target_modules as $module) {
      $this->registerNewTargetModule($module);
    }

    $old_target_modules = array_diff($current_target_modules, $modules);
    foreach ($old_target_modules as $module) {
      $this->unregisterOldTargetModule($module);
    }
    drupal_static_reset('getTargetModules');
  }

  /**
   * Return the default target module.
   */
  public function getDefaultTargetModule() {
    return $this->configFactory->get('config_enforce_devel.settings')->get('defaults')['target_module'];
  }

  /**
   * Ensure that there is a default target module installed and registered.
   */
  public function ensureDefaultTargetModule() {
    // If there is already a default target module, there is nothing to do here.
    if ($this->getDefaultTargetModule()) return;

    $target_modules = $this->getTargetModules();
    // If there is at least one target module, default to the first one.
    if (!empty($target_modules)) {
      return $this->saveDefaultTargetModule(array_shift($target_modules));
    }

    $module_name = 'config_enforce_default';
    (new TargetModule())
      ->setName($this->t('Config Enforce - Default Target Module'))
      ->setMachineName($module_name)
      ->setDescription($this->t('This is a default target module created when Config Enforce Devel is installed for the first time. It can be deleted safely once a new target module has been created.'))
      ->setPath('modules/custom')
      ->createModule()
      ->installModule()
      ->registerAsTargetModule();

    return $this->saveDefaultTargetModule($module_name);
  }

  /**
   * Initialize enforced configs for a given target module.
   */
  protected function registerNewTargetModule($module) {
    (new TargetModule())
      ->setName(ConfigEnforce::getAllModules()[$module])
      ->setMachineName($module)
      ->initializeEnforcedConfigs();
  }

  /**
   * Delete enforced configs for a given target module.
   */
  protected function unregisterOldTargetModule($module) {
    // We cannot unregister the default target module without causing undesired behaviour.
    if ($module == $this->getDefaultTargetModule()) return;
    (new EnforcedConfigCollection($module))->delete();
  }

  /**
   * Ensure that there is a default target module installed and registered.
   */
  protected function saveDefaultTargetModule(string $module) {
    $settings = $this->configFactory->getEditable('config_enforce_devel.settings');
    $defaults = $settings->get('defaults');
    $defaults['target_module'] = $module;
    $settings->set('defaults', $defaults)->save();

    $message = $this->t('Set default target module to %module.', [
      '%module' => $module,
    ]);

    $this->log->notice($message);
    $this->messenger()->AddMessage($message);

    // Reload the page, so that the default target module can apply.
    return header("Refresh:0");
  }

}
